@@ -6,73 +6,105 @@ use utf8;
 use Archive::Zip          qw/AZ_OK COMPRESSION_LEVEL_DEFAULT/;
 use Scalar::Util          qw/looks_like_number blessed/;
 use List::Util            qw/none/;
-use Params::Validate      qw/validate_with SCALAR SCALARREF UNDEF/;
 use POSIX                 qw/strftime/;
 use Date::Calc            qw/Delta_Days/;
 use Carp                  qw/croak/;
 use Encode                qw/encode_utf8/;
+use Data::Domain 1.16     qw/:all/;
+use Try::Tiny;
 
-our $VERSION = '1.05';
+our $VERSION = '1.09';
 
 #======================================================================
 # GLOBALS
 #======================================================================
 
-my $DATE_STYLE = 1;                        # 0-based index into the <cellXfs> format for dates ..
-                                           # .. defined in the styles() method
-
-my $SHEET_NAME = qr(^[^\\/?*\[\]]{1,31}$); # valid sheet names: <32 chars, no chars \/?*[] 
-my $TABLE_NAME = qr(^\w{3,}$);             # valid table names: >= 3 chars, no spaces
-
-
-# specification in Params::Validate format for checking parameters to the new() method 
-my %params_spec = (
-
-  # date_regex : for identifying dates in data cells. Should capture into $+{d}, $+{m} and $+{y}.
-  date_regex        => {type => SCALARREF|UNDEF, optional => 1, default =>
-                         qr[^(?: (?<d>\d\d?)    \. (?<m>\d\d?) \. (?<y>\d\d\d\d)  # dd.mm.yyyy
-                               | (?<y>\d\d\d\d) -  (?<m>\d\d?) -  (?<d>\d\d?)     # yyyy-mm-dd
-                               | (?<m>\d\d?)    /  (?<d>\d\d?) /  (?<y>\d\d\d\d)) # mm/dd/yyyy
-                             $]x},
-
-  # bool_regex : for identifying booleans in data cells. If true, should capture into $1
-  bool_regex        => {type => SCALARREF|UNDEF, optional => 1, default => qr[^(?:(TRUE)|FALSE)$]},
-
-  compression_level => {type => SCALAR, regex => qr/^\d$/, optional => 1, default => COMPRESSION_LEVEL_DEFAULT},
-
- );
+my $DATE_STYLE = 1;                           # 0-based index into the <cellXfs> format for dates ..
+                                              # .. defined in the styles() method
 
+my $RX_SHEET_NAME = qr(^[^\\/?*\[\]]{1,31}$); # valid sheet names: <32 chars, no chars \/?*[] 
+my $RX_TABLE_NAME = qr(^\w{3,}$);             # valid table names: >= 3 chars, no spaces
 
 my %entity       = ( '<' => '&lt;', '>' => '&gt;', '&' => '&amp;' );
 my $entity_regex = do {my $chars = join "", keys %entity; qr/[$chars]/};
 
+#======================================================================
+# SIGNATURES FOR CONTROLLING ARGS TO PUBLIC METHODS
+#======================================================================
 
+my $sig_for_new = Struict( # Struict = strict Struct .. not a typo!
+
+  # date_regex : for identifying dates in data cells. Should capture into $+{d}, $+{m} and $+{y}.
+  date_regex        => Regexp(-if_absent =>
+                         qr[^(?: (?<d>\d\d?)    \. (?<m>\d\d?) \. (?<y>\d\d\d\d)  # dd.mm.yyyy
+                               | (?<y>\d\d\d\d) -  (?<m>\d\d?) -  (?<d>\d\d?)     # yyyy-mm-dd
+                               | (?<m>\d\d?)    /  (?<d>\d\d?) /  (?<y>\d\d\d\d)) # mm/dd/yyyy
+                             $]x),
+
+  # bool_regex : for identifying booleans in data cells. If true, should capture into $1
+  bool_regex        => Regexp(-if_absent => qr[^(?:(TRUE)|FALSE)$]),
+
+  # ZIP compression level
+  compression_level => Int(-range => [1, 9], -if_absent => COMPRESSION_LEVEL_DEFAULT),
+ )->meth_signature;
+
+
+my $sig_for_add_sheet = List(
+  String(-name => 'sheet_name', -regex => $RX_SHEET_NAME),
+  String(-name => 'table_name', -regex => $RX_TABLE_NAME, -optional => 1),
+  List  (-name => 'headers',    -all => String,           -optional => 1),
+  One_of(-name => 'rows_maker', -options => [List,                                      # an array of rows, or
+                                             Coderef,                                   # a row generator coderef, or
+                                             Obj(-isa => 'DBI::st'),                    # a DBI statement, or
+                                             Obj(-isa => 'DBIx::DataModel::Statement'), # a DBIx::DataModel statement
+                                            ]),
+  Struict(-name => 'options', -optional => 1, -fields => {
+      cols => One_of(List(-all => Num),
+                     List(-all => Struct(width => Num(-optional => 1),
+                                         style => Int(-optional => 1),
+                                         min   => Int(-optional => 1),
+                                         max   => Int(-optional => 1))))
+    }),
+ )->meth_signature;
+
+my $sig_for_add_sheets_from_database = List(-items => [Obj   (-isa     => 'DBI::db'),
+                                                       String(-default => "S.")],
+                                            -all   => String,
+ )->meth_signature;
+
+my $sig_for_add_defined_name = List(String(-name => "name"),
+                                    String(-name => "formula"),
+                                    String(-name => "comments", -optional => 1),
+  )->meth_signature;
+
+
+
+my $sig_for_save_as = One_of(String,
+                             Whatever(-does => 'IO'),
+  )->meth_signature;
+
+
+  
 #======================================================================
 # CONSTRUCTOR
 #======================================================================
 
 sub new {
-  my $class = shift;
+  my ($class, %self) = &$sig_for_new;
 
-  # check parameters and create $self
-  my $self = validate_with( params      => \@_,
-                            spec        => \%params_spec,
-                            allow_extra => 0,
-                           );
-
-  # initial values for internal data structures
-  $self->{sheets}                = []; # array of sheet names
-  $self->{tables}                = []; # array of table names
-  $self->{shared_string}         = {}; # ($string => $string_index)
-  $self->{n_strings_in_workbook} = 0;  # total nb of strings (including duplicates)
-  $self->{last_string_id}        = 0;  # index for the next shared string
-  $self->{defined_names}         = {}; # ($name => [$formula, $comment])
+  # initial values for internal data structures (constructor args cannot initialize those)
+  $self{sheets}                = []; # array of sheet names
+  $self{tables}                = []; # array of table names
+  $self{shared_string}         = {}; # ($string => $string_index)
+  $self{n_strings_in_workbook} = 0;  # total nb of strings (including duplicates)
+  $self{last_string_id}        = 0;  # index for the next shared string
+  $self{defined_names}         = {}; # ($name => [$formula, $comment])
 
   # immediately open a Zip archive
-  $self->{zip} = Archive::Zip->new;
+  $self{zip} = Archive::Zip->new;
 
   # return the constructed object
-  bless $self, $class;
+  bless \%self, $class;
 }
 
 
@@ -82,66 +114,103 @@ sub new {
 
 
 sub add_sheet {
-  # 3rd parameter ($headers) may be omitted -- so we insert an undef if necessary
-  splice @_, 3, 0, undef if @_ < 5;
+  # the 3rd parameter ($headers) may be omitted -- so we insert an undef if necessary
+  splice @_, 3, 0, undef if @_ < 5 or @_ == 5 && (ref $_[4] // '') eq 'HASH';
 
   # now we can parse the parameters
-  my ($self, $sheet_name, $table_name, $headers, $rows_maker) = @_;
+  my ($self, $sheet_name, $table_name, $headers, $rows_maker, $options) = &$sig_for_add_sheet;
 
-  # check if the given sheet name is valid
-  $sheet_name =~ $SHEET_NAME
-    or croak "'$sheet_name' is not a valid sheet name";
+  # register the sheet name 
   none {$sheet_name eq $_} @{$self->{sheets}}
     or croak "this workbook already has a sheet named '$sheet_name'";
+  push @{$self->{sheets}}, $sheet_name;
 
-  # local copies for convenience
-  my $date_regex = $self->{date_regex};
-  my $bool_regex = $self->{bool_regex};
+  # iterator for generating rows
+  my $row_iterator = $self->_build_row_iterator($rows_maker, \$headers);
+
+  # build inner XML
+  my ($xml, $last_row, $last_col) = $self->_build_rows($headers, $row_iterator, $table_name);
+
+  # add XML preamble and close sheet data
+  my $preamble = join "", 
+    q{<?xml version="1.0" encoding="UTF-8" standalone="yes"?>},
+    q{<worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main"},
+              q{ xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">};
+  $preamble .= $self->_xml_for_options($options) if $options;
+  $preamble .= q{<sheetData>};
+  substr $xml, 0, 0, $preamble;
+  $xml .= q{</sheetData>};
+
+  # if required, add the table corresponding to this sheet into the zip archive, and refer to it in XML
+  my @table_rels;
+  if ($table_name && $headers) {
+    my $table_id = $self->_add_table($table_name, $last_col, $last_row, @$headers);
+    push @table_rels, $table_id;
+    $xml .= q{<tableParts count="1"><tablePart r:id="rId1"/></tableParts>};
+  }
+
+  # close the worksheet xml
+  $xml .= q{</worksheet>};
+
+  # insert the sheet and its rels into the zip archive
+  my $sheet_id   = $self->n_sheets;
+  my $sheet_file = "sheet$sheet_id.xml";
+  $self->add_string_to_zip(encode_utf8($xml),                  "xl/worksheets/$sheet_file"           );
+  $self->add_string_to_zip($self->worksheet_rels(@table_rels), "xl/worksheets/_rels/$sheet_file.rels");
+
+  return $sheet_id;
+}
+
+sub _build_row_iterator {
+  my ($self, $rows_maker, $headers_ref) = @_;
+
+  my $iterator;
 
-  # iterator for generating rows; either received as argument or built as a closure upon an array
-  my $next_row;
   my $ref = ref $rows_maker;
   if ($ref && $ref eq 'CODE') {
-    $next_row  = $rows_maker;
-    $headers //= $next_row->();
+    $iterator  = $rows_maker;
+    $$headers_ref //= $iterator->();
   }
   elsif ($ref && $ref eq 'ARRAY') {
     my $i = 0;
-    $next_row  = sub { $i < @$rows_maker ? $rows_maker->[$i++] : undef};
-    $headers //= $next_row->();
+    $iterator  = sub { $i < @$rows_maker ? $rows_maker->[$i++] : undef};
+    $$headers_ref //= $iterator->();
   }
   elsif (blessed $rows_maker && $rows_maker->isa('DBI::st')) {
     $rows_maker->{Executed}
       or croak '->add_sheet(..., $sth) : the statement handle must be executed (call the $sth->execute method)';
-    $next_row  = sub { $rows_maker->fetchrow_arrayref};
-    $headers //= $rows_maker->{NAME}; # see L<DBI>
+    $iterator  = sub { $rows_maker->fetchrow_arrayref};
+    $$headers_ref //= $rows_maker->{NAME}; # see L<DBI>
   }
   elsif (blessed $rows_maker && $rows_maker->isa('DBIx::DataModel::Statement')) {
-    $headers //= $rows_maker->sth->{NAME};
-    $next_row  = sub {my $row = $rows_maker->next; return $row ? [@{$row}{@$headers}] : ()};
+    DBIx::DataModel->VERSION >= 3.0
+      or croak 'add_sheet(.., $statement) : requires DBIx::DataModel >= 3.0; your version is ', DBIx::DataModel->VERSION;
+    $$headers_ref //= $rows_maker->sth->{NAME};
+    $iterator  = sub {my $row = $rows_maker->next; return $row ? [@{$row}{@$$headers_ref}] : ()};
   }
   else {
     croak 'add_sheet() : missing or invalid last argument ($rows_maker)';
   }
 
+  return $iterator;
+}
+
+sub _build_rows {
+  my ($self, $headers, $row_iterator, $table_name) = @_;
+
+  my $xml = "";
+
+  # local copies for convenience
+  my $date_regex = $self->{date_regex};
+  my $bool_regex = $self->{bool_regex};
+
   # array of column references in A1 Excel notation
   my @col_letters = ('A'); # this array will be expanded on demand in the loop below
 
-  # register the sheet name 
-  push @{$self->{sheets}}, $sheet_name;
-
-  # start building XML for the sheet
-  my @xml = (
-    q{<?xml version="1.0" encoding="UTF-8" standalone="yes"?>},
-    q{<worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main"},
-              q{ xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">},
-    q{<sheetData>},
-    );
-
   # loop over rows and columns
   my $row_num = 0;
  ROW:
-  for (my $row = $headers; $row; $row = $next_row->()) {
+  for (my $row = $headers; $row; $row = $row_iterator->()) {
     $row_num++;
     my $last_col = @$row or next ROW;
     my @cells;
@@ -149,64 +218,78 @@ sub add_sheet {
   COLUMN:
     foreach my $col (0 .. $last_col-1) {
 
-      # if this column letter is not known yet, compute it using Perl's increment op on strings
+      # if this column letter is not known yet, compute it using Perl's increment op on strings (so 'AA' comes after 'Z')
       my $col_letter = $col_letters[$col]
                    //= do {my $prev_letter = $col_letters[$col-1]; ++$prev_letter};
 
       # get the value; if the cell is empty, no need to write it into the XML
       my $val = $row->[$col];
       defined $val and length $val or next COLUMN;
+      my $n_days; # in case we need to parse a date
 
       # choose XML attributes and inner value
       # NOTE : for perl, looks_like_number( "INFINITY") is TRUE! Hence the test $val !~ /^\pL/
-      (my $tag, my $attrs, $val)
-        = looks_like_number($val) && $val !~ /^\pL/ ? (v => ""                  , $val                          )
-        : $date_regex && $val =~ $date_regex        ? (v => qq{ s="$DATE_STYLE"}, n_days($+{y}, $+{m}, $+{d})   )
-        : $bool_regex && $val =~ $bool_regex        ? (v => qq{ t="b"}          , $1 ? 1 : 0                    )
-        : $val =~ /^=/                              ? (f => "",                   escape_formula($val)          )
-        :                                             (v => qq{ t="s"}          , $self->add_shared_string($val));
+      (                                              my $tag,  my $attrs,            $val)
+      #                                                 ====   =========             ====
+        = looks_like_number($val) && $val !~ /^\pL/    ? (v => ""                  , $val                           )
+        : $date_regex && $val =~ $date_regex
+                      && is_valid_date(\%+, \$n_days)  ? (v => qq{ s="$DATE_STYLE"}, $n_days                        )
+        : $bool_regex && $val =~ $bool_regex           ? (v => qq{ t="b"}          , $1 ? 1 : 0                     )
+        : $val =~ /^=/                                 ? (f => "",                   escape_formula($val)           )
+        :                                                (v => qq{ t="s"}          , $self->_add_shared_string($val));
 
       # add the new XML cell
-      my $cell = sprintf qq{<c r="%s%d"%s><%s>%s</%s></c>}, $col_letter, $row_num, $attrs, $tag, $val, $tag;
+      my $cell = qq{<c r="$col_letter$row_num"$attrs><$tag>$val</$tag></c>};
       push @cells, $cell;
     }
 
     # generate the row XML and add it to the sheet
     my $row_xml = join "", qq{<row r="$row_num" spans="1:$last_col">}, @cells, qq{</row>};
-    push @xml, $row_xml;
+    $xml .= $row_xml;
   }
 
-  # close sheet data
-  push @xml, q{</sheetData>};
+  # if this sheet contains an Excel table, make sure there is at least one data row
+  ++$row_num and $xml .= qq{<row r="$row_num" spans="1:1"></row>}
+    if $table_name && $row_num == 1;
 
-  # if required, add the table corresponding to this sheet into the zip archive, and refer to it in XML
-  my @table_rels;
-  if ($table_name && $row_num) {
-    my $table_id = $self->add_table($table_name, $col_letters[-1], $row_num, @$headers);
-    push @table_rels, $table_id;
-    push @xml, q{<tableParts count="1"><tablePart r:id="rId1"/></tableParts>};
-  }
-
-  # close the worksheet xml
-  push @xml, q{</worksheet>};
-
-  # insert the sheet and its rels into the zip archive
-  my $sheet_id   = $self->n_sheets;
-  my $sheet_file = "sheet$sheet_id.xml";
-  $self->{zip}->addString(encode_utf8(join("", @xml)),
-                          "xl/worksheets/$sheet_file",
-                          $self->{compression_level});
-  $self->{zip}->addString($self->worksheet_rels(@table_rels),
-                          "xl/worksheets/_rels/$sheet_file.rels",
-                          $self->{compression_level});
-
-  return $sheet_id;
+  return ($xml, $row_num, $col_letters[-1]);
 }
 
 
 
+sub _xml_for_options {
+  my ($self, $options) = @_;
+
+  # currently there is only one option 'cols'. Handled below in a separate sub for better clarity.
+  return $self->_xml_for_cols_option($options->{cols});
+}
+
+
+sub _xml_for_cols_option {
+  my ($self, $cols) = @_;
+
+  my $xml = "<cols>";
+  my $next_col_num = 1;
+  foreach my $col (@$cols) {
+    # build attributes for the node
+    my %attrs = ref $col ? %$col : (width => $col); # cols => [6, ...] is just syntactic sugar for => [{witdh => 6}, ...]
+    $attrs{$_} //= $next_col_num for qw/min max/;   # colrange to which this <col> specification applies
+    $attrs{customWidth} //= 1 if $attrs{width};     # tells Excel that the width is not automatic
+
+    # generate XML from attributes
+    $xml .= join(" ", "<col", map {qq{$_="$attrs{$_}"}} keys %attrs) . "/>";
+
+    # compute index of next column
+    $next_col_num = $attrs{max} + 1;
+  }
+  $xml .= "</cols>";
+
+  return $xml;
+}
+
+
 sub add_sheets_from_database {
-  my ($self, $dbh, $sheet_prefix, @table_names) = @_;
+  my ($self, $dbh, $sheet_prefix, @table_names) = &$sig_for_add_sheets_from_database;
 
   # in absence of table names, get them from the database metadata
   if (!@table_names) {
@@ -214,8 +297,6 @@ sub add_sheets_from_database {
     @table_names = map {$_->{TABLE_NAME}} @$tables;
   }
 
-  $sheet_prefix //= "S.";
-
   foreach my $table (@table_names) {
     my $sth = $dbh->prepare("select * from $table");
     $sth->execute;
@@ -225,7 +306,7 @@ sub add_sheets_from_database {
 
 
 
-sub add_shared_string {
+sub _add_shared_string {
   my ($self, $string) = @_;
 
   # single quote before an initial equal sign is ignored (escaping the '=' like in Excel)
@@ -240,16 +321,12 @@ sub add_shared_string {
 
 
 
-sub add_table {
+sub _add_table {
   my ($self, $table_name, $last_col, $last_row, @col_names) = @_;
 
-  # check if the given table name is valid
-  $table_name =~ $TABLE_NAME
-    or croak "'$table_name' is not a valid table name";
+  # register this table
   none {$table_name eq $_} @{$self->{tables}}
     or croak "this workbook already has a table named '$table_name'";
-
-  # register this table
   push @{$self->{tables}}, $table_name;
   my $table_id = $self->n_tables;
 
@@ -274,18 +351,15 @@ sub add_table {
    );
 
   # insert into the zip archive
-  $self->{zip}->addString(encode_utf8(join "", @xml),
-                          "xl/tables/table$table_id.xml",
-                          $self->{compression_level});
+  $self->add_string_to_zip(encode_utf8(join "", @xml), "xl/tables/table$table_id.xml");
 
   return $table_id;
 }
 
 
 sub add_defined_name {
-  my ($self, $name, $formula, $comment) = @_;
+  my ($self, $name, $formula, $comment) = &$sig_for_add_defined_name;
 
-  $name && $formula                        or croak 'add_defined_name($name, $formula): empty argument'; 
   not exists $self->{defined_names}{$name} or croak "add_defined_name(): name '$name' already in use";
   $self->{defined_names}{$name} = [$formula, $comment];
 }
@@ -305,26 +379,33 @@ sub worksheet_rels {
 #======================================================================
 
 sub save_as {
-  my ($self, $target) = @_;
+  my ($self, $target) = &$sig_for_save_as;
 
   # assemble all parts within the zip, except sheets and tables that were already added previously
-  my $zip = $self->{zip};
-  $zip->addString($self->content_types,  "[Content_Types].xml"        , $self->{compression_level});
-  $zip->addString($self->core,           "docProps/core.xml"          , $self->{compression_level});
-  $zip->addString($self->app,            "docProps/app.xml"           , $self->{compression_level});
-  $zip->addString($self->workbook,       "xl/workbook.xml"            , $self->{compression_level});
-  $zip->addString($self->_rels,          "_rels/.rels"                , $self->{compression_level});
-  $zip->addString($self->workbook_rels,  "xl/_rels/workbook.xml.rels" , $self->{compression_level});
-  $zip->addString($self->shared_strings, "xl/sharedStrings.xml"       , $self->{compression_level});
-  $zip->addString($self->styles,         "xl/styles.xml"              , $self->{compression_level});
+  $self->add_string_to_zip($self->content_types,  "[Content_Types].xml"       );
+  $self->add_string_to_zip($self->core,           "docProps/core.xml"         );
+  $self->add_string_to_zip($self->app,            "docProps/app.xml"          );
+  $self->add_string_to_zip($self->workbook,       "xl/workbook.xml"           );
+  $self->add_string_to_zip($self->_rels,          "_rels/.rels"               );
+  $self->add_string_to_zip($self->workbook_rels,  "xl/_rels/workbook.xml.rels");
+  $self->add_string_to_zip($self->shared_strings, "xl/sharedStrings.xml"      );
+  $self->add_string_to_zip($self->styles,         "xl/styles.xml"             );
 
   # write the Zip archive
-  my $write_result = ref $target ? $zip->writeToFileHandle($target) : $zip->writeToFileNamed($target);
+  my $write_result = ref $target ? $self->{zip}->writeToFileHandle($target) : $self->{zip}->writeToFileNamed($target);
   $write_result == AZ_OK
     or croak "could not save Zip archive into " . (ref($target) || $target);
 }
 
 
+sub add_string_to_zip {
+  my ($self, $content, $name) = @_;
+
+  $self->{zip}->addString($content, $name, $self->{compression_level});
+}
+
+
+
 sub _rels {
   my ($self) = @_;
 
@@ -569,15 +650,24 @@ sub escape_formula {
 }
 
 
-sub n_days {
-  my ($y, $m, $d) = @_;
+sub is_valid_date {
+  my ($named_captures, $n_days_ref) = @_;
+  my ($y, $m, $d) = @{$named_captures}{qw/y m d/};
 
-  # convert the given date into a number of days since 1st January 1900
-  my $n_days = Delta_Days(1900, 1, 1, $y, $m, $d) + 1;
-  my $is_after_february_1900 = $n_days > 59;
-  $n_days += 1 if $is_after_february_1900; # because Excel wrongly treats 1900 as a leap year
+  # years before 1900 can't be handled by Excel
+  return undef if $y < 1900;
 
-  return $n_days;
+  # convert the given date into a number of days since 1st January 1900.
+  my $return_status = try   {$$n_days_ref = Delta_Days(1900, 1, 1, $y, $m, $d) + 1;
+                             my $is_after_february_1900 = $$n_days_ref > 59;
+                             $$n_days_ref += 1 if $is_after_february_1900; # because Excel wrongly treats 1900 as a leap year
+                             1; # success
+                            };
+                             # no catch .. undef if failure (invalid date)
+ 
+  return $return_status;
+
+  # NOTE : invalid dates will be inserted as Excel strings
 }
 
 
@@ -660,7 +750,8 @@ A compiled regular expression for detecting data cells that contain dates.
 The default implementation recognizes dates in C<dd.mm.yyyy>, C<yyyy-mm-dd>
 and C<mm/dd/yyyy> formats. User-supplied regular expressions should use
 named captures so that the day, month and year values can be found respectively
-in C<< $+{d} >>, C<< $+{m} >> and C<< $+{y} >>.
+in C<< $+{d} >>, C<< $+{m} >> and C<< $+{y} >>. If this parameter receives C<undef>,
+no date handling will be performed.
 
 =item bool_regex
 
@@ -671,6 +762,8 @@ parenthesis so that the content is captured in C<$1>. Here is an example for fre
 
   $writer = Excel::ValueWriter::XLSX->new(bool_regex => qr[^(?:(VRAI)|FAUX)$]);
 
+If this parameter receives C<undef>, no bool handling will be performed.
+
 
 =item compression_level
 
@@ -683,7 +776,7 @@ L<Archive::Zip>, which amounts to 6.
 
 =head2 add_sheet
 
-  $writer->add_sheet($sheet_name, $table_name, [$headers,] $rows_maker);
+  $writer->add_sheet($sheet_name, $table_name[, $headers], $rows_maker[, \%options]);
 
 Adds a new worksheet into the workbook.
 
@@ -743,16 +836,45 @@ an executed L<DBI> statement handle
 
 =item *
 
-a L<DBIx::DataModel::Statement> object
+a L<DBIx::DataModel::Statement> object (version 3.0 or greater)
 
 =back
 
+=item *
+
+C<< \%options >> is an optional reference to a hash of additional options.
+Currently the only option is C<cols>, which points to an arrayref of column specifications.
+The arrayref may contain :
+
+=over
+
+=item *
+
+a list of numbers specifying the width of successive columns, like for example :
+
+  $writer->add_sheet(S_sales => sales => [qw/date product_id descr price/], $rows, {cols => [12, 8, 20, 8]);
+
+Column widths in Excel are expressed in "character units", i.e. a requested width of 8 should
+be just wide enough to display the string '00000000' (8 zero characters displayed with the normal font).
+
+=item *
+
+a list of hashrefs specifying column attributes. The C<width> attribute corresponds to column widths,
+as explained above. The C<style> attribute is a numeric index into the list of styles defined within
+the workbook -- except that currently this module only supports one single style for dates, so this
+addition has no practical use at the moment.
+
+=back
+
+
 =back
 
 Cells within a row must contain scalar values. Values that look like numbers are treated
 as numbers. String values that match the C<date_regex> are converted into numbers and
-displayed through a date format. String values that start with an initial '=' are treated
-as formulas; but like in Excel, if you want regular string that starts with a '=', put a single
+displayed through a date format, but only if the date is valid and is above 1900 January 1st --
+otherwise it is treated as a string, like in Excel.
+String values that start with an initial '=' are treated
+as formulas; but like in Excel, if you want a plain string that starts with a '=', put a single
 quote just before the '=' -- that single quote will be removed from the string.
 Everything else is treated as a string. Strings are shared at the workbook level
 (hence a string that appears several times in the input data will be stored
